/*
	PRIMITIVE.H
	Dichiarazione delle primitive di disegno
	Tutte sono derivate da un oggetto "primitiva generica". Essa e' derivata da
	CObject e contiene le caratteristiche in comune per tutte le primitive.


	Note sull'aggiunta di nuove primitive
	=====================================
	Quando si aggiungono nuove primitive, occorre inserire codice in
	(ordine logico):

	-Primitive.h e Primitive.cpp, ovviamente;
	-stdafx.h, enumerazione SELECTED_PRIMITIVE;
	-stdafx.h, define di NUM_PRIMITIVES;
	-CElecDrawView::OnLButtonDown (creazione di nuova primitiva);
	-CPrimitive::ScanLine (caricamento);
	-Risorsa toolbar: disegno del pulsante;
	-CMainFrame::OnButtonXXX (selezione della primitiva attiva).
*/


//////////////////////////////////////////////////////////////////
// Classe CPrimitive

class CPrimitive;
typedef CArray<CPrimitive*,CPrimitive*>		PRIMARRAY;
typedef CArray<CString*,CString*>			STRARRAY;
class SUBDIR { public: CString name; int index; };
typedef CArray<SUBDIR,SUBDIR>				SUBDIRARRAY;


class CPrimitive: public CObject
{
	public:
		CPrimitive();
		~CPrimitive();

		// Comandi
		virtual void MoveRel( CPoint ptrel);
		virtual void Rotate( CPoint pt, BOOL bCounterClockWise=FALSE);
		virtual void Swap( int xPos);	// L'asse di swap e' verticale
		virtual void UpdateBoundingRgn( CDC *pdc);

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();
		virtual BOOL operator= (CPrimitive& src);

		// Proprieta'
		virtual PROXIMITY_TYPE IsPtNear( CPoint pt, CDC *pdc);
		virtual BOOL IsProper( void);
		virtual void SaveState( void);
		virtual void LoadState( void);
		virtual CPoint GetPoint( int nIndex);
		virtual void SetPoint( int nIndex, CPoint pt, BOOL bSignal=FALSE);
		virtual void Select( BOOL bSel = TRUE);
		virtual BOOL IsSelected( void);
		virtual CRgn* GetRegion( void);

		virtual void SetConst( BOOL bConst = TRUE);
		virtual BOOL IsConst( void);
		virtual int GetDrawPriority( void);

/*
	Impostazione dinamica degli attributi
	Funzioni per l'impostazione "modulare" degli attributi.

	GetAttribName:
->	index: indice dell'attributo
<-	name: nome restituito dell'attributo
<-	values: possibili valori dell'attributo (vedi sotto)
	La funzione restituisce FALSE per indici non definiti, per indicare che non ci sono piu`
	attributi.

	GetAttrib:
->	name: nome dell'attributo di cui leggere il valore
<-	value: valore dell'attributo
	La funzione restituisce FALSE se l'attributo non e` disponibile.

	SetAttrib:
->	name: nome dell'attributo da impostare
->	value: valore da assegnare
	La funzione restituisce FALSE se il nome dell'attributo o il valore sono invalidi.


	Note sul formato di 'values'
	----------------------------
	Per ora sono supportati questi formati:
	"#INT" -> numero intero qualunque
	"#INT n m" -> numero intero da n a m
	"#STRING" -> stringa
	"(stringa1 stringa2 ...)" -> scelta "obbligata" fra le stringhe elencate (separate da spazi)

	In ogni caso nel parametro 'value' c'e` sempre una stringa; e` compito della funzione membro
	(o del chiamante) convertire opportunamente al valore target. Se la conversione non e` possibile,
	SetAttrib deve restituire FALSE.
*/

		virtual BOOL GetAttribName( int index, CString& name, CString& values) { return FALSE; };
		virtual BOOL GetAttrib( const char *name, CString& value) { return FALSE; };
		virtual BOOL SetAttrib( const char *name, const char *value) { return FALSE; };

		// Helper functions
		static void CleanBuffer( CString& buffer);

		static DOC_TYPE GetType( CString& buffer);

		static CString ExtractString( CString& linea, int nOggettiPrima);

		static CPrimitive* ScanLine( CString& str);

		static BOOL ScanBuffer( CString& str, PRIMARRAY &ar, BOOL *bWarning);

		static BOOL ScanClipboard( PRIMARRAY &ar, CWnd* MainWnd, BOOL* bWarning);

		static BOOL ScanHeadedBuffer( CString& str, PRIMARRAY *ar, STRARRAY *lm,
			CString *title, BOOL *bWarning, DOC_TYPE *t, SUBDIRARRAY* sda=NULL);

		static BOOL ScanFile( CString& filename, PRIMARRAY *ar, STRARRAY *lm, CString *title,
			BOOL *bWarning, DOC_TYPE *t, CString* ExternalBuffer=NULL);

		static BOOL RegioneLinea( CDC* dc, CRgn& rgn, CPoint pt1, CPoint pt2, int largh);

		static void Rimpiazza( CString& str, const char *src, const char *dest);

		static void InitLastSettings();

		// Funzioni virtuali da implementare (anche se non sono pure)

/*
	Nota sul parametro 'zoom' della funzione Draw(): esso serve solo ed esclusivamente per
	il disegno "immediato" della primitiva, e non influisce minimamente su nient'altro.
	L'unico motivo per cui e` stato introdotto e` per migliorare la risoluzione dell'output
	dei metafile, che al 100% vengono sgranati. Ma non serve a nulla impostare il mapping
	mode per i metafile DC, dato che le coordinate considerate sono quelle logiche.
*/
		virtual void Draw( CDC *pdc, BOOL bXNOR=FALSE, COLORREF backcolor=RGB(255,255,255),
			COLORREF forecolor=RGB(0,0,0), COLORREF selcolor=RGB(255,0,0), int zoom=100, int nLayer=-1,
			BOOL blackandwhite=FALSE) {};
		virtual int GetNumDefPoints( void) { return 0; };
		virtual DWORD GetProp( void) { return 0; };
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE) { return FALSE; };
		virtual void Serialize( CArchive& ar) {};

		virtual int SetLayer( int layer) { int oldlayer = m_nLayer; if( layer>=0 && layer<16) m_nLayer = layer; return oldlayer; };
		virtual int GetLayer() { return m_nLayer; };

	protected:
		CPoint		m_ptDefining[MAX_DEFINING_POINTS+1];
		BOOL		m_bSelected;
		int			m_nRotations;
		CRgn		m_Region;
		BOOL		m_bSwapped;

		// Proprieta' comuni
		int			m_nLayer;
		BOOL		m_bConst;

		// Backup
		CPoint		m_ptSaved[MAX_DEFINING_POINTS+1];
		int			m_nSavedRotations;
		BOOL		m_bSavedSwapped;
};

//////////////////////////////////////////////////////////////////
// Classe CLine

class CLine: public CPrimitive
{
	DECLARE_SERIAL( CLine)

	public:
		CLine();
		CLine( CPoint start, CPoint stop=CPoint(-1,-1), BOOL bConst = FALSE);
		~CLine();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
};


//////////////////////////////////////////////////////////////////
// Classe CExtendedText (non e` utile derivarla da CText, perche`
// tanto la maggior parte delle funzioni membro sono da riscrivere)

#define EXTFONT_STYLE_BOLD			0x00000001
#define EXTFONT_STYLE_ITALIC		0x00000002
#define EXTFONT_STYLE_MIRRORED		0x00000004
#define EXTFONT_STYLE_UNDERLINED	0x00000008

class CExtendedText: public CPrimitive
{
	friend CPrimitive;

	DECLARE_SERIAL( CExtendedText)

	public:
		CExtendedText();
		CExtendedText( CPoint start, CString text, CString face="", int height=-1,
			int width=-1, int flags=-1, int angle=-1, BOOL bConst = FALSE);

		~CExtendedText();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();
		virtual BOOL operator= (CExtendedText& src);

		// Impostazione dinamica degli attributi
		virtual BOOL GetAttribName( int index, CString& name, CString& values);
		virtual BOOL GetAttrib( const char *name, CString& value);
		virtual BOOL SetAttrib( const char *name, const char *value);

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual void Rotate( CPoint pt, BOOL bCounterClockWise=FALSE);
		virtual void Swap( int xPos);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		static CPrimitive* Load( CString& str);
		BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
		CFont	m_Font;			// Sarebbe stupido ricrearlo tutte le volte
		CString m_sText;
		CString m_FontFace;		// Il nome del font (solo non proporzionali!)
		int		m_FontHeight;	// Altezza font (in unita` logiche)
		int		m_FontWidth;	// Larghezza font (idem)
		DWORD	m_Flags;		// bold, italic, ecc...
		int		m_Angle;		// Angolo di inclinazione

		static CString TrovaFontFissi();
		static int CALLBACK EnumFontFamExProc( ENUMLOGFONTEX *lpelfe,
			NEWTEXTMETRICEX *lpntme, int FontType, LPARAM lParam);
		BOOL RecreateFont( int zoom=100);
		void GetBoundingPoly( POINT *pts, CSize& size, int zoom=100, BOOL bNoSwap=FALSE);
		CRect GetBoundingPolyRect( POINT *pts);
};


//////////////////////////////////////////////////////////////////
// Classe CConnection

class CConnection: public CPrimitive
{
	DECLARE_SERIAL( CConnection)

	public:
		CConnection();
		CConnection( CPoint start, BOOL bConst = FALSE);
		~CConnection();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
};


//////////////////////////////////////////////////////////////////
// Classe CBezier

class CBezier: public CPrimitive
{
	DECLARE_SERIAL( CBezier)

	public:
		CBezier();
		CBezier( CPoint p1, CPoint p2=CPoint(-1,-1), CPoint p3=CPoint(-1,-1),
			CPoint p4=CPoint(-1,-1), BOOL bConst = FALSE);
		~CBezier();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual int GetNumDefPoints( void);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual DWORD GetProp( void);
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
};


//////////////////////////////////////////////////////////////////
// Classe CPoly

class CPoly: public CPrimitive
{
	friend CPrimitive;

	DECLARE_SERIAL( CPoly)

	public:
		CPoly();
		CPoly( CPoint p1, CPoint p2=CPoint(-1,-1), CPoint p3=CPoint(-1,-1),
			BOOL bConst = FALSE);
		~CPoly();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();
		virtual BOOL operator= (CPoly& src);

		// Impostazione dinamica degli attributi
		virtual BOOL GetAttribName( int index, CString& name, CString& values);
		virtual BOOL GetAttrib( const char *name, CString& value);
		virtual BOOL SetAttrib( const char *name, const char *value);

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual int GetNumDefPoints( void);
		virtual void SetPoint( int nIndex, CPoint pt, BOOL bSignal=FALSE);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual DWORD GetProp( void);
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
		int		m_nVirtualPts;
		BOOL	m_bFilled;
};


//////////////////////////////////////////////////////////////////
// Classe CEllipse

class CEllipse: public CPrimitive
{
	friend CPrimitive;

	DECLARE_SERIAL( CEllipse)

	public:
		CEllipse();
		CEllipse( CPoint start, CPoint stop=CPoint(-1,-1), BOOL bConst = FALSE);
		~CEllipse();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();
		virtual BOOL operator= (CEllipse& src);

		// Impostazione dinamica degli attributi
		virtual BOOL GetAttribName( int index, CString& name, CString& values);
		virtual BOOL GetAttrib( const char *name, CString& value);
		virtual BOOL SetAttrib( const char *name, const char *value);

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		virtual void UpdateBoundingRgn( CDC *pdc);
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
		BOOL m_bFilled;
};


//////////////////////////////////////////////////////////////////
// Classe CRectangle

class CRectangle: public CPrimitive
{
	friend CPrimitive;

	DECLARE_SERIAL( CRectangle)

	public:
		CRectangle();
		CRectangle( CPoint start, CPoint stop=CPoint(-1,-1), BOOL bConst = FALSE);
		~CRectangle();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();
		virtual BOOL operator= (CRectangle& src);

		// Impostazione dinamica degli attributi
		virtual BOOL GetAttribName( int index, CString& name, CString& values);
		virtual BOOL GetAttrib( const char *name, CString& value);
		virtual BOOL SetAttrib( const char *name, const char *value);

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		virtual void UpdateBoundingRgn( CDC *pdc);
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
		BOOL m_bFilled;
};


//////////////////////////////////////////////////////////////////
// Classe CMacro

class CMacro: public CPrimitive
{
	DECLARE_SERIAL( CMacro)

	public:
		CMacro();
		CMacro( CPoint pt);
		~CMacro();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();
		virtual BOOL operator= (CMacro& src);

		// Impostazione dinamica degli attributi
		virtual BOOL GetAttribName( int index, CString& name, CString& values);
		virtual BOOL GetAttrib( const char *name, CString& value);
		virtual BOOL SetAttrib( const char *name, const char *value);

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite=FALSE);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		virtual int GetDrawPriority( void);

		virtual void SetPoint( int nIndex, CPoint pt, BOOL bSignal=FALSE);
		virtual void Rotate( CPoint pt, BOOL bCCV=FALSE);
		virtual void Swap( int xPos);
		virtual void MoveRel( CPoint ptrel);
		virtual void LoadState( void);
		virtual void SaveState( void);
		virtual void Select( BOOL bSel = TRUE);

		// Impostare un layer per una macro e` illegale
		virtual int SetLayer( int layer) { return 0; };

		// Macro-specific
		virtual PRIMARRAY* GetComponentArray( void);
		static CPrimitive* Load( CString& str);
		void Adjust( void);

		// Serializzazione
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

		// Helper
		static BOOL ScanLibraryBuffer( CString& buffer, STRARRAY& lm, SUBDIRARRAY* sda=NULL);

		static BOOL ScanLibraryMacro( CString& buffer, CMacro* dest, CString* titleShort,
			CString* titleLong, BOOL* bWarning);

		static BOOL ScanMacroName( CString& macroname, CMacro& dest, BOOL *bWarning,
			BOOL *bStandard);

	private:
		PRIMARRAY	m_Objects;
		CString		m_sText;
};

//////////////////////////////////////////////////////////////////
// Classe CPCBLine

class CPCBLine: public CPrimitive
{
	friend CPrimitive;

	DECLARE_SERIAL( CPCBLine)

	public:
		CPCBLine();
		CPCBLine( CPoint start, int thickness=-1, CPoint stop=CPoint(-1,-1), BOOL bConst = FALSE);
		~CPCBLine();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();

		// Impostazione dinamica degli attributi
		virtual BOOL GetAttribName( int index, CString& name, CString& values);
		virtual BOOL GetAttrib( const char *name, CString& value);
		virtual BOOL SetAttrib( const char *name, const char *value);

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		virtual BOOL IsProper();
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);

	private:
		int m_thickness;
};

//////////////////////////////////////////////////////////////////
// Classe CPCBPad

class CPCBPad: public CPrimitive
{
	friend CPrimitive;

	DECLARE_SERIAL( CPCBPad)

	public:

		enum PadType
		{
			PAD_ROUND,
			PAD_SQUARE,
			PAD_SQUARE_ROUNDED
		};

		CPCBPad();
		CPCBPad( CPoint start, int sizex=-1, int sizey=-1,
			int intdiam=-1, int style=-1, BOOL bConst = FALSE);
		~CPCBPad();

		// Operatori e funzioni di clonazione
		virtual CPrimitive* Clone();

		// Impostazione dinamica degli attributi
		virtual BOOL GetAttribName( int index, CString& name, CString& values);
		virtual BOOL GetAttrib( const char *name, CString& value);
		virtual BOOL SetAttrib( const char *name, const char *value);

		// Funzioni di disegno e modifica
		virtual void Draw( CDC *pdc, BOOL bXNOR, COLORREF backcolor,
			COLORREF forecolor, COLORREF selcolor, int zoom, int nLayer, BOOL blackandwhite);
		virtual void UpdateBoundingRgn( CDC *pdc);
		virtual int GetNumDefPoints( void);
		virtual DWORD GetProp( void);
		void Rotate( CPoint pt, BOOL bCCV);
		virtual BOOL IsProper();
		static CPrimitive* Load( CString& str);
		virtual BOOL Save( CString& str, BOOL bSplit=FALSE);
		virtual void Serialize( CArchive& ar);
		virtual int GetDrawPriority( void);

	private:
		int			m_sizex;
		int			m_sizey;
		int			m_intdiam;
		int			m_style;
};
